#include <inttypes.h>
#include "i2c.h"
#include <avr/io.h>

// define i2c_debug

// Mask the TWSR
#define TWSRM (TWSR & 0xF8)

uint8_t I2C_flags, I2C_counter;
#define init 1
#define control 2
#define max_count 0x80



unsigned char I2C_Init(void){
	TWBR = I2C_TWBR;

	TWSR |= (I2C_PRESCALER<<TWPS0);
	
	TWCR |= (1<<TWEN); //enable
	I2C_flags |= (1<<init);
	
	I2C_counter = max_count;
	
	return 1;
}

unsigned char I2C_Start(void){
	
	if(!bit_is_set(I2C_flags, init)) return -1;
	//loop_until_bit_is_clear(TWCR, TWINT); //ensure that no data is corrupted
	/*sbi(TWCR, TWSTA); //Request Start
	sbi(TWCR, TWINT); //Reset Int.*/
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);
	
	while (!(TWCR & (1<<TWINT))) {
		if (I2C_counter>0) {
			I2C_counter--;
		} else {
			I2C_counter = max_count;
			return -1;
		}
	}
	/*Here the software may hang up, if there is some Problem 
	with the I2c - if no response within max_count cycles --> 
	EXIT with error*/
	
	return TWSRM;

}


unsigned char I2C_Stop(void){
	//loop_until_bit_is_clear(TWCR, TWINT); //ensure that no data is corrupted
	/*sbi(TWCR, TWSTO); //set stop
	sbi(TWCR, TWINT); //start stop*/
	TWCR = (1<<TWINT) | (1<<TWSTO);
	while (!(TWCR & (1<<TWINT))) {
		if (I2C_counter>0) {
			I2C_counter--;
		} else {
			I2C_counter = max_count;
			return -1;
		}
	}
	return TWSRM;
}

unsigned char I2C_Write(unsigned char Data){
	//loop_until_bit_is_clear(TWCR, TWINT); //ensure that no data is corrupted
	TWDR = Data;
	TWCR = (1<<TWINT) | (1<<TWEN);//sbi(TWCR,TWINT);
	while (!(TWCR & (1<<TWINT)));
	return TWSRM;
}

unsigned char I2C_Read(unsigned char *data, int8_t Stop){
	//loop_until_bit_is_clear(TWCR, TWINT); //ensure that no data is corrupted
	/*if (Stop == 1) sbi(TWCR, TWSTO);
	else cbi(TWCR, TWSTO);
	cbi(TWCR, TWSTA);*/
	if (Stop == 1) {
		TWCR = (1<<TWINT) | (1<<TWEN) ;
	} else{
		TWCR = (1<<TWINT) | (1<<TWEN) | (1 << TWEA);
	}
	while (!(TWCR & (1<<TWINT)));
	*data = TWDR;
	return TWSRM;
}



unsigned char I2C_Readdata(unsigned char add,unsigned char *data, uint8_t length){
	unsigned char tmp;
	uint8_t i;
	if (length == 0) return -1; // add Error
	tmp = I2C_Start();
	if((tmp != 0x08) & (tmp != 0x10)) {
		return tmp;
	}

	add |= (1<<0); //read
	if ((tmp = I2C_Write(add)) != 0x40){
		return tmp;
	}	

	length --;
	for(i = 0; i < length; i++){
		if ((tmp = I2C_Read(&data[i], 0)) != 0x50){
			return tmp;
		}
	}
	if((tmp = I2C_Read(&data[i],1)) != 0x58){
		return tmp;
	}

	tmp = I2C_Stop();
	
	return 1;
}

unsigned char I2C_Readdata1(unsigned char add, unsigned char *data){
	return   I2C_Readdata(add, data, 1);
}


unsigned char I2C_Writedata (unsigned char add,unsigned char *data, uint8_t length){
	unsigned char tmp;
	uint8_t i;
	if (length == 0) return -1 ;// add Error
	add &= ~(1<<0); //write
	tmp = I2C_Start();
	if((tmp != 0x08) & (tmp != 0x10)){
		return tmp;
	}

	
	if ((tmp = I2C_Write(add)) != 0x18 ){
		return tmp;
	}

	for(i = 0; i < length; i++){
		if ((tmp = I2C_Write(data[i])) != 0x28){
			return tmp;
		}
	}

	tmp = I2C_Stop();
	
	return 1;
}
 
unsigned char I2C_Writedata1(unsigned char add, unsigned char data){
	return I2C_Writedata(add, &data, 1);
}
