/*
Die Routine l2a() wandelt die Zahl 'l' in eine ASCII-Zeichenkette, die in
s[12] abgelegt wird: Vorzeichen, max. 10 Ziffern, Abschluss -> 12 Byte.

Zeitaufwendige Divisionen werden durch Subtraktionen ersetzt, was deutlich
schneller ausgefhrt wird.

Bei Bedarf kann 'long' durch 'int' ersetzt werden; hierdurch wird der
Code reduziert, die Ausfuerung beschleunig aber der Wertebereich auf
-32768 bis 32767 begrenzt.
Nach Anpassung kann diese Routine auch dazu verwendet werden, die Mantisse
von float Zahlen sehr schnell zu wandeln.

Michael Nowak, 2004-11-09, www.mino-elektronik.de
*/
#include "l2a.h"

void l2a(long l, char *s)       // l -> zu wandelnde Zahl, *s -> Ergebnis
{
// diese Tabelle sollte im flash liegen !
unsigned long tab[] = {1000000000, 100000000, 10000000, 1000000,
		       100000, 10000, 1000, 100, 10, 1, 0};
unsigned long temp;             // fuer schnellere Ausfuehrung
char i=0,j;                     // i -> tab-index, j -> aktuelle Ziffer
  if(l) {			// wert ist != 0
    if(l < 0) {
      l = -l;
      *s++ = '-';               // wenn negativ
    }
    while(tab[i] > l) i++;	// 1. gueltige Ziffer suchen, Vornullen entfernen
    do {
      temp = tab[i];            // temp vermeidet wiederholte Indizierung
      j = '0';			// Startwert = Ziffer '0'
      if(temp) {                // nur wenn gueltige Tabellenwerte
	for(;j <= '9'; j++) {   // mehr als '9' darf's nicht werden
	  l -= temp;		// solange subtrahieren
	  if(l < 0) {           // bis < 0: das war einmal zu viel
	    l += temp;          // daher wieder korrigieren
	    break;              // for-Schleife ist fertig
	  }
	}
	*s++ = j;               // Ziffer ablegen
	i++;                    // naechst kleinere Stelle auswerten
      }
    } while(temp);		// bis Tabelle abgearbeitet
  } else *s++ = '0';            // falls Zahl == 0, nur eine '0' erzeugen
  *s = 0;			// string abschlieen
}